#include <timers.h>
#include "povS1.h"
//prototypes des gestionnaires d'interruption
void InterruptHandlerHigh (void);
void InterruptHandlerLow (void);
void BinToLeds( unsigned int value) ;
void DisplaySpeed( unsigned int n0 );

extern volatile unsigned int LedsArray[60] ;
extern volatile unsigned char LowSpeed ;
extern unsigned int max_it ;
unsigned int n0, n0f , n3 , last_index ,delta = 0  ;
char Ovf = 0  ;
unsigned int tabn0[10],j=0,k,sn0;
long alpha = LOW_PASS_ALPHA ;
unsigned char hour=3,min=0,sec=0; // 03:00:00 au POR
unsigned char NItTimer1=0;

//Placement d'un saut vers le gestionnaire d'interruption
//----------------------------------------------------------------------------
// High priority interrupt vector
#if defined(BOOTLOADABLE)
   #pragma code InterruptVectorHigh = 0x1008
#else
   #pragma code InterruptVectorHigh = 0x08
#endif
void 
InterruptVectorHigh (void)
{
  _asm
    goto InterruptHandlerHigh //jump to interrupt routine
  _endasm
}
// Low priority interrupt vector
#if defined(BOOTLOADABLE)
 #pragma code InterruptVectorLow = 0x1018
#else
 #pragma code InterruptVectorLow = 0x18
#endif
void
InterruptVectorLow (void)
{
  _asm
    goto InterruptHandlerLow //jump to interrupt routine
  _endasm
}
// Routines gestionnaire d'interruption
//----------------------------------------------------------------------------
// High priority interrupt routine
#pragma code
#pragma interrupt InterruptHandlerHigh
void
InterruptHandlerHigh ()
{
    static unsigned int i , clr=0 , nit120 ;
    unsigned int j , sync = 0 ;
     // IT 50ms timer1 : gestion du temps et de LedsArray[]
    // pour 1s on compte 1s/50ms = 20 it
    if ( PIR1bits.TMR1IF) {

        WriteTimer1(3036) ;
        PIR1bits.TMR1IF = 0 ;
        NItTimer1++;
        if (NItTimer1 >=max_it  ) { // 5it pour tx4 !!
            NItTimer1 = 0 ;
           // LedsArray[sec]=0;
            LedsArray[min]=0;
            LedsArray[hour*5]=0;         
             if ( sec < 59 ) { sec++;}
             else {
                 sec = 0 ;
                 sync = 1 ;
                 for (j=0;j<60;j++)
                 { LedsArray[j]=0x0; }
                 if ( min < 59 ) { min++; }
                 else {
                     min = 0 ;
                     if ( hour < 11) { hour++ ; }
                     else { hour = 0 ; }
                 }
             } 
            // Maj heure avant minute pour le cas o aiguilles confondues !
            LedsArray[hour*5]=0x0FF0;  
            LedsArray[min]=0x0FFE;
           
           for (j=0;j<60;j+=5){
                 LedsArray[j]|=1;
            }
            // on allume la led au bout de l'aiguille des heures
            // lorsque les secondes dpassent la position des heures
            if ( sec >=(hour*5) ) {
                LedsArray[hour*5]|=1 ;
            }
            // idem pour le bout de l'aiguille des minutes
            if ( sec >= min ) {
                LedsArray[min]|=1;
            }
            LedsArray[sec]|=1; // Rempli le bord extrieur
        }              
    }
   // IT overflow timer 0
   // si cette IT apparait on tourne trop lentement
    if ( INTCONbits.TMR0IF) {

        LowSpeed = 1 ;
        Ovf = 1 ;
        n0=0;
        INTCONbits.TMR0IF = 0 ;
    }

    
    // IT REF 0 tour
    if ( INTCONbits.INT0IF  ) {
       
         // on lit la vitesse du tour prcdent
         n0 = ReadTimer0();
         
         // Filtrage basse pas sur n0 
         // n0f = (1-a)*n0f + a*n0  avec a=0,05
         // pour viter les flottants on multiplie tout par 2^16 
         // ensuite un dcalage de 16 bits est trs efficace
         // par dfaut : a = 0,05 = 3277 / 65536 et 1=65536/65536
         // a-> 0 filtrage fort , a-> 1 filtrage faible
         n0f = ((65536L - alpha)*n0f + alpha*n0)>>16 ;
         #if defined(HARDWARE_VERSION_BEFORE_2013)
         // Timer3 va 32 fois plus vite que timer0 et on veut une it tous
         // les 3 donc 120 fois par tour 120=360/3
         n3 = (n0f*32L)/120;
         #else
         // Timer3 va 16 fois plus vite que timer0 et on veut une it tous
         // les 3 donc 120 fois par tour 120=360/3
         n3 = (n0f*16L)/120;

         #endif
         last_index = nit120 ;
         nit120=0;
          
         // le dernier index doit tre 119 s'il y a 120 it par tour
         // on ajuste n3 pour qu'il en soit ainsi
         if ( last_index < 119) { delta+=10; }
         else if (last_index > 119 ) { delta-=10;}
         // si on a le 0 avant 119 on est en retard donc n3 diminue
         n3 -= delta ;
       
        if ( n0 > 60000) { //  TODO : crer Macro en fonction de FOSC
            LowSpeed = 1 ;

        } else {
            LowSpeed = 0 ;
                   }

        WriteTimer0(0); // RAZ timer0;

        WriteTimer3(0xFFFF - n3);
        BinToLeds(LedsArray[0]);
        
        i=1;
        clr=1;
        PIR2bits.TMR3IF = 0 ;
        INTCONbits.INT0IF = 0 ;
  
      }

    // IT 1/120 tour
    if ( PIR2bits.TMR3IF ) {
       nit120++; // compte it  3
       if (!LowSpeed) {
       // une fois sur deux (6):
           // soit on sort LedsArray sur les Leds
           // soit on teint tout
        if (!clr){
             BinToLeds(LedsArray[i]);
             if ( i< 59 )  i++;  else i=0 ;
             }
        else
           {
               BinToLeds(0);
           }
        clr = 1 - clr ;
        }

       WriteTimer3(0xFFFF - n3) ;
       PIR2bits.TMR3IF = 0 ;
    }
   
    
}

extern volatile char msg[MAX_MSG+1];
extern volatile unsigned char rx ;
// Low priority interrupt routine
#pragma code
#pragma interrupt InterruptHandlerLow
void
InterruptHandlerLow ()
{

  static char i ; 
   char c ;
  // Partie rception d'un caractre
  if(PIR1bits.RCIF) // si un car arriv
    {
      c=ReadUSART(); // le lire
      WriteUSART(c); // echo
      if (c==CAR_LF) return ; // ignore les LF
      if( c==CAR_BS)
        {
         if(i>0){
             i--;  }
        }
      else {
        if(c!=CAR_CR && i<MAX_MSG)
         {
           msg[i++]=c ; // stockage
         }else
         {
           msg[i]='\0'; // fin de chane si CR
           i=0;
           rx=1;
         }
      }
    }
}

