#include "povS1.h"
#include "utils.h"
#include "ledsaccess.h"
// Ici on trouve les portions de codes qui dpend du hardware.
// Ils y a seulement :
//  Init_leds : positionne les broches des ports o se trouvent les
//  Leds en sorties
//  BinToLeds : applique une valeur binaire sur les leds
//  bit0 -> led0 , bit1-> led1 , .... , bit8 -> led9 max pour 2550
//  pour 2520 on utilise en plus les bits 9,10,11 pour les trois leds
//  suplmentaires. Comme les valeurs sont unsigned on peut les dcaler
//   droites sans risque.

#if defined(HARDWARE_VERSION_BEFORE_2013) && defined (__18F2520)
void Init_leds(void) {

    ClearAllLeds () ;
    // Leds output
    TRISLED0to1  =  LED0to1_OUTPUT ;
    TRISLED2to7 =  LED2to7_OUTPUT ;
    TRISLED8to11 = LED8to11_OUTPUT ;
}

// display a binary value on leds
void BinToLeds( unsigned int value) {

    LED0to1   =  (value & 0x0003) << 4 ;
    LED2to7  =   (value & 0x00FC) >> 2 ;
    LED8to11 =   (value & 0x0F00) >> 6 ;

}
#elif defined(HARDWARE_VERSION_2013) && defined (__18F2550)
void Init_leds(void) {

    ClearAllLeds () ;

    // Leds output
    TRISLED0to1  =  LED0to1_OUTPUT ;
    TRISLED2to4 =   LED2to4_OUTPUT ;
    TRISLED5to8 =   LED5to8_OUTPUT ;
}

// display a binary value on leds
void BinToLeds( unsigned int value) {

    LED0to1   =   (value & 0x0003) << 4 ;
    LED2to4   =   (value & 0x001C) >> 2 ;
    LED5to8   =   (value & 0x01E0) >> 3 ;

}
#elif defined(HARDWARE_VERSION_2013) && defined (__18F2520)

void Init_leds(void) {

    ClearAllLeds () ;
     
    // Leds output
    TRISLED0to1  =  LED0to1_OUTPUT ;
    TRISLED2to4and9to11 =  LED2to4and9to11_OUTPUT ;
    TRISLED5to8 = LED5to8_OUTPUT ;
}

// display a binary value on leds
void BinToLeds( unsigned int value) {


    LED0to1   =  (value & 0x0003) << 4 ;

    // Led 2,3,4 -> RC0,RC1,RC2
    // Led 11 -> RC3
    // Led 9,10 -> RC4,RC5
    LED2to4and9to11 =    (value & 0x001C) >> 2   \
                      |  (value & 0x0800) >> 8   \
                      |  (value & 0x0600) >> 5 ;

    
    // Led 5,6,7,8 -> RB2,3,4,5
    LED5to8 =   (value & 0x01E0) >> 3 ; 

}
#endif


// display a decimal value on leds
void DecToLeds(unsigned char value) {

    if ( value == 0 || value > MAXLED) {
         BinToLeds(0);
         return ; }

    BinToLeds( (unsigned int)1<<(value-1) );

}

void ClearAllLeds () {

    BinToLeds(0);
}

void SetAllLeds() {

    BinToLeds(4095);

}

// Allume n leds
void LedOnx(unsigned char n) {
    unsigned int v=0 ;
    while(n--) {
        v <<= 1 ;
        v |=  1 ;
    }
    BinToLeds(v);
}

// fonctions d'essais des leds
void DoLedsTest(void) {

    int  i;
    for (i=0;i<MAXLED;i++) {
        DecToLeds(i+1);
        Delay1MSx(300);
    }
    for (i--;i>=0;i--) {
        DecToLeds(i+1);
        Delay1MSx(300);
    }


}
void DoLedsTest2(void) {

    int  i;
    for (i=0;i<MAXLED;i++) {
        LedOnx(i+1);
        Delay1MSx(300);
    }
    for (i--;i>=0;i--) {
        LedOnx(i+1);
        Delay1MSx(300);
    }


}