/* Project : Programm for POVS1 USB board (rev 2.4.1)
 * File:   main.c
 * Author: Philippe Arlotto
 *
 * Created on 4 juin 2012, 11:26
 */


#include "povS1.h"
#include "ledsaccess.h"
#include "bluetooth.h"
#include "analog.h"
#include "utils.h"


#if defined(__18F2520) && defined(HARDWARE_VERSION_BEFORE_2013)
#pragma config OSC = HSPLL  //  Fquartz = 10 MHz Fosc=4xFquartz=40MHz
#pragma config WDT = OFF
#pragma config LVP = OFF
#pragma config PBADEN = OFF

#elif defined(__18F2550)
        #pragma config PLLDIV   = 5    // (20 MHz crystal on POV USB board)
        #pragma config CPUDIV   = OSC1_PLL2
        #pragma config USBDIV   = 2         // Clock source from 96MHz PLL/2
        //#pragma config FOSC     = HSPLL_HS // CPU osc = 48MHz (?)
        #pragma config FOSC     = HS // CPU osc = 20MHz
        #pragma config FCMEN    = OFF
        #pragma config IESO     = OFF
        #pragma config PWRT     = OFF
        #pragma config BOR      = ON
        #pragma config BORV     = 3
        #pragma config VREGEN   = ON		//USB Voltage Regulator
        #pragma config WDT      = OFF
        #pragma config WDTPS    = 32768
        #pragma config MCLRE    = ON
        #pragma config LPT1OSC  = OFF
        #pragma config PBADEN   = OFF
        #pragma config LVP = OFF
        #pragma config STVREN   = ON
#elif defined(__18F2520) && defined(HARDWARE_VERSION_2013)

#pragma config OSC = HS  //  Fquartz = 20 MHz Fosc=Fquartz=20MHz
#pragma config WDT = OFF
#pragma config LVP = OFF
#pragma config PBADEN = OFF

#endif

#if defined(__18F2550) && defined(BOOTLOADABLE)
// Pour utiliser le Bootloader HID de Microchip
#pragma code REMAPPED_RESET_VECTOR=0x1000
extern void _startup (void);

void _reset (void)
{
   _asm goto _startup _endasm
}

#endif

#ifdef BOOTLOADABLE
#warning "Attention Version bootloadable utiliser le bootloader HID pour la charger "
#endif

#pragma code

void init ( void ) ;

void SwSetTime(void) ;

unsigned int t = 0 ;  // Compteur de tour de tche de fond
unsigned char sw ; // tat des switch (0-3)
unsigned int max_it = 20 ; 
unsigned int vbat ; // tension batterie en mV
volatile unsigned char LowSpeed = 1 ; // Indique vitesse trop basse pour POV
volatile unsigned int LedsArray[60] = {0}; // Etat des leds tous les 6
unsigned char x = 0 , sec0 ;
int temp ;
volatile char msg[MAX_MSG+1];
volatile unsigned char rx = 0 ;

extern unsigned char hour,min,sec ; // heure mise  jour dans interrup.c


void main(void) {

    init();
    sw = ReadSwitches();
    DecToLeds(sw);
    if ( sw == 2) {
       // temps acclr si BP2 appuys au POR
        max_it = 5 ;   // temps x 4
    }
   
  //  while (sw=ReadSwitches()); // attend fin appuie btn

    printf(VERSION"\r\n$");
     // Autorise interruptions
    RCONbits.IPEN = 1; // priorits actives
    INTCONbits.GIEH = 1 ;
    INTCONbits.GIEL = 1 ;
    while (1)
    {

       // affiche temprature tant que BP2 appuy
       while (ReadSwitches()==2)  {
            temp = ReadTemp()/10 ; // conversion en C
            temp -= 18 ; // on affiche  partir de 18C
            ClearAllLeds () ;
            if (temp>=0) {   
                LedOnx(temp/2); // 2C/LED
            }          
        }

        if (ReadSwitches()==3) {
             sec0 = sec ; // test entre mode rglage heure
            while(ReadSwitches()==3) {
                if (sec > (sec0+3)%60) { 
                    SetAllLeds();
                    SwSetTime();
                }
            }
        }
        if ( rx ) {
            LED7 = !LED7 ; // test rx
            execute_command();
            rx = 0 ;
        }
        // Si vitesse trop basse LED8 reflte tat du phototransistor
        if (LowSpeed) {
            LED8 = REF_INPUT ;
        }
        t++; 
        if ( t>60000) {
            t=0;
            // affiche vbat et led 0 clignotante tant que vitesse trop basse
            if (LowSpeed) {
                 vbat = ReadVBAT();
                 LedOnx((vbat/1000)/2);
                 THERM_ON = 0 ; // pour ne pas chauffer la thermistance
                 LED0 = x;
                 x = 1-x;
            }
             
           
        }
    }
}


void init ( void ) {
    
    Init_leds();

    // ADC : vref+ =5 , vref- = 0 , AN0-AN3 analogiques
    OpenADC(ADC_FOSC_64 &
        ADC_RIGHT_JUST &
        ADC_16_TAD  ,
        ADC_CH0 &
        ADC_INT_OFF &
        ADC_VREFPLUS_VDD &
        ADC_VREFMINUS_VSS,
        11 ); // AN0-AN3 analogs other digitals
     // Interruption REF active : index 0
    INTCON2bits.INTEDG0 = 0 ; // Interrupt on falling edge
    INTCONbits.INT0E = 1 ;
    // Timer 0 : mesure dure d'un tour :
     #if defined(HARDWARE_VERSION_BEFORE_2013)
    // Kdiv = 32 Toverflow = 32.2^16.0.1us = 209,72ms  40Mhz
    // 209,7ms/tour <=> 286 tr/min Overflow si V<286tr/min
    // Pour 500 tr/min = 120ms/tour N0timer =  37500
    OpenTimer0( TIMER_INT_ON  &
                T0_16BIT   &
                T0_SOURCE_INT &
                T0_PS_1_32
                 ) ;
    #else
    // Kdiv = 16 Toverflow = 32.2^16.0.2us = 209,72ms  20Mhz
    // 209,7ms/tour <=> 286 tr/min Overflow si V<286tr/min
    // Pour 500 tr/min = 120ms/tour N0timer =  37500
    OpenTimer0( TIMER_INT_ON  &
                T0_16BIT   &
                T0_SOURCE_INT &
                T0_PS_1_16
                 ) ;

    #endif
    INTCON2bits.TMR0IP = 1 ; // timer 0 prio haute

    // Timer 3 : 120 it par tour soit tous les 3 (360/120)
    // A 500 tr/min 3 <=> 1ms
    // A 2000 tr/min 3 <=> 0.25ms
    // T3Overflow = 1x2^16x0.1us = 6,55ms
    OpenTimer3(TIMER_INT_ON &
               T3_16BIT_RW  &
               T3_SOURCE_INT &
               T3_PS_1_1    &
               T3_SYNC_EXT_OFF );
    IPR2bits.TMR3IP=1 ; // timer3 prio haute

    // Timer 1 : horloge
     #if defined(HARDWARE_VERSION_BEFORE_2013)
     //T1Overflow  = 8x2^16x0.1us = 52,24ms sans prchargement
      //50ms = 8xNx0.1u <=> N = 62500 prchargement  65536-62500 = 3036
     OpenTimer1(TIMER_INT_ON &
                T1_16BIT_RW  &
                T1_SOURCE_INT &
                T1_PS_1_8    &
                T1_SYNC_EXT_OFF &
                T1_OSC1EN_OFF );
    #else
     //T1Overflow  = 4x2^16x0.2us = 52,24ms sans prchargement
    //50ms = 4xNx0.2u <=> N = 62500 prchargement  65536-62500 = 3036
     OpenTimer1(TIMER_INT_ON &
                T1_16BIT_RW  &
                T1_SOURCE_INT &
                T1_PS_1_4    &
                T1_SYNC_EXT_OFF &
                T1_OSC1EN_OFF );
    #endif
     WriteTimer1(3036);

     IPR1bits.TMR1IP = 1 ; // timer1 prio haute
  #if defined(HARDWARE_VERSION_BEFORE_2013)
    // init 9600Bd  40Mhz avec it en rx
    OpenUSART( USART_TX_INT_OFF &
                USART_RX_INT_ON &
                USART_ASYNCH_MODE &
                USART_EIGHT_BIT &
                USART_CONT_RX &
                USART_BRGH_LOW,
                64 );
  #else
    // init 9600Bd  20Mhz avec it en rx
    OpenUSART( USART_TX_INT_OFF &
                USART_RX_INT_ON &
                USART_ASYNCH_MODE &
                USART_EIGHT_BIT &
                USART_CONT_RX &
                USART_BRGH_HIGH,
                129 );

  #endif

    IPR1bits.RCIP = 0 ; // uart prio basse



    SetAllLeds() ;
    Delay1SECx(1) ;
    ClearAllLeds () ;
    Delay1SECx(1) ;
    DoLedsTest();
}




#if MAXLED == 12

void SwSetTime(void) {
   unsigned char done = 0 , state = 0 , btn ;
   unsigned char new_hour,new_min,new_sec ;
   unsigned char new_min_d , new_min_u ;
   unsigned char i ;
   Delay1SECx(1) ;
   ClearAllLeds () ;
   
   new_hour = hour ;
   new_min = min ;
   new_min_d = new_min / 10 ;
   new_min_u = new_min % 10 ;
   new_sec = 0 ;
   INTCONbits.GIEH = 0 ;
   while ( !done )
   {
       if ( ( btn=ReadSwitches()) ==3 ) done = 1 ;
       switch(state) {
           case 0 : // Rglage de l'heure
               DecToLeds(new_hour);
               LED11=LED_ON ;
               if ( btn==1 ) {
                   if ( new_hour < 11 ) { new_hour++; }
                   else new_hour=0;
                   state = 1 ;
               } else {
                   if (btn == 2) state = 22 ;
               }
               break ;
           case 1 :
               if ( btn == 0 ) state = 0 ;
               break ;
            case 22 :
               if ( btn == 0 ) state = 2 ;
               break ;
           case 2 : // Rglage des dizaines de minutes
               DecToLeds(new_min_d);
               LED11=LED_OFF ;
               LED10=LED_ON ;
               if ( btn==1 ) {
                   if ( new_min_d < 5 ) { new_min_d++; }
                   else new_min_d=0;
                   state = 3 ;
               } else {
                   if (btn == 2) state = 44 ;
               }
               break ;
           case 3 :
               if ( btn == 0 ) state = 2 ;
               break ;
           case 44 :
               if ( btn == 0 ) state = 4 ;
               break ;
           case 4 : // Rglage des units de minutes
               DecToLeds(new_min_u);
               LED11=LED_ON ;
               LED10=LED_ON ;
               if ( btn==1 ) {
                   if ( new_min_u < 9 ) { new_min_u++; }
                   else new_min_u=0;
                   state = 5 ;
               } else {
                   if (btn == 2) state = 66 ;
               }
               break ;
           case 5 :
               if ( btn == 0 ) state = 4 ;
               break ;
           case 66 :
               if ( btn == 0 ) state = 6 ;
               break ;
           case 6 : // Attente top dpart
               ClearAllLeds () ;
               BinToLeds(0X555);
               hour = new_hour ;
               min = 10 * new_min_d + new_min_u ;
               sec = 0 ;
               if ( btn == 1) {
               WriteTimer1(3036);
               INTCONbits.GIEH = 1 ;
               // MAJ LedsArray
                for (i=0;i<60;i+=5){
                 LedsArray[i]=0;
               }
               for (i=0;i<60;i+=5){
                 LedsArray[i]|=1;
               }
               LedsArray[min]=0x0FFE;
               LedsArray[hour*5]=0x0FF0;
               done =1 ;
               }
               break ;
           default : // erreur !!
               hour = min =sec = 0 ;
               WriteTimer1(3036);
               INTCONbits.GIEH = 1 ;
               done =1 ;
       }
    Delay10KTCYx( 100 ) ; // debounce delay
   }
}
#else
 void SwSetTime(void){} // TODO !!
#endif
