#ifndef __POV_H
#define __POV_H
#include <stdio.h>
#include <stdlib.h>
#include <p18cxxx.h>
#include <delays.h>
#include <timers.h>
#include <adc.h>
#include <usart.h>

#define VERSION "GEII POV CLOCK V2.6 built : "__DATE__" "__TIME__"\r\n"




// system defs
#if defined(HARDWARE_VERSION_BEFORE_2013)
 #define OSC_FREQ 40E6 // Qz=10Mhz x4 (HSPLL)
#else
 #define OSC_FREQ 20E6 // Qz=20Mhz (HS)
#endif

#define SYS_TCY  (4./OSC_FREQ)
#define SECOND_TCY   (1./SYS_TCY)
#define SECOND_1OKTCY ( SECOND_TCY / 10E3 )
#define MS_1KTCY (1E-3/(SYS_TCY)/1000)
// Led defs
// dfinitions indpendantes de la version hardware
#define LED0 LATAbits.LATA4
#define LED1 LATAbits.LATA5
#define THERM_ON LED1 // Led1 commande aussi la tension sur RTH

#define LED0to1  LATA
#define TRISLED0to1  TRISA
#define LED0to1_OUTPUT 0b11001111

// dfinitions dpendantes de la version hardware et du processeur
#if defined(HARDWARE_VERSION_BEFORE_2013) && defined (__18F2520)

#define MAXLED 12

#define LED2 LATCbits.LATC0
#define LED3 LATCbits.LATC1
#define LED4 LATCbits.LATC2
#define LED5 LATCbits.LATC3
#define LED6 LATCbits.LATC4  
#define LED7 LATCbits.LATC5


#define LED2to7 LATC
#define TRISLED2to7 TRISC
#define LED2to7_OUTPUT 0b11000000

#define LED8 LATBbits.LATB2
#define LED9 LATBbits.LATB3
#define LED10 LATBbits.LATB4
#define LED11 LATBbits.LATB5

#define LED8to11 LATB
#define TRISLED8to11 TRISB
#define LED8to11_OUTPUT 0b11000011


#elif defined(HARDWARE_VERSION_2013) && defined (__18F2550)

#define MAXLED 9

#define LED2 LATCbits.LATC0
#define LED3 LATCbits.LATC1
#define LED4 LATCbits.LATC2

#define LED2to4 LATC
#define TRISLED2to4 TRISC
#define LED2to4_OUTPUT 0b11111000

#define LED5 LATBbits.LATB2
#define LED6 LATBbits.LATB3
#define LED7 LATBbits.LATB4
#define LED8 LATBbits.LATB5 

#define LED5to8 LATB
#define TRISLED5to8 TRISB
#define LED5to8_OUTPUT 0b11000011

#elif defined(HARDWARE_VERSION_2013) && defined (__18F2520)
 
    #define MAXLED 12

    // Leds 2,3,4 et 9,10,11  PORTC
    #define LED2 LATCbits.LATC0
    #define LED3 LATCbits.LATC1
    #define LED4 LATCbits.LATC2

    #define LED9 LATCbits.LATC4
    #define LED10 LATCbits.LATC5
    #define LED11 LATCbits.LATC3

    #define LED2to4and9to11 LATC
    #define TRISLED2to4and9to11 TRISC
    #define LED2to4and9to11_OUTPUT 0b11000000

    // Leds 5,6,7,8  PORTB
    #define LED5 LATBbits.LATB2
    #define LED6 LATBbits.LATB3
    #define LED7 LATBbits.LATB4
    #define LED8 LATBbits.LATB5

    #define LED5to8 LATB
    #define TRISLED5to8 TRISB
    #define LED5to8_OUTPUT 0b11000011

#else
 #error "Version hardware non dfinie ou incompatible avec le processeur choisi"
#endif


#define LED_OFF 0
#define LED_ON 1

// logical inputs defs
#define BP1 PORTBbits.RB1
#define REF_INPUT PORTBbits.RB0
#define BP2 REF_INPUT
#define SW_PORT PORTB

// analog inputs defs
#include  <adc.h>
#define CH_TEMP ADC_CH3
#define CH_MVBAT ADC_CH2
#define MVBAT_RATIO 381L //  regler par une mesure
#define MVBAT_FACTOR 100L

// uart defines
#define MAX_MSG 20
#define CAR_CR 0x0D
#define CAR_LF 0x0A
#define CAR_BS 0x08

// coefficient alpha du filtre passe bas sur la vitesse
//  alpha = 0,05 = 3277 / 65536
#define LOW_PASS_ALPHA 3277L

#endif